; PSMFILL.LSP 
; Copyright 1995 by Looking Glass Microproducts
; Apply the PSFILL command to selected entities.

(setq PSMFILL:PATTERN "." ; last fill pattern used
      PSMFILL:PARAMS  nil ; PostScript Fill Pattern Parameters
)
(defun C:PSMFILL (/ OLD_ERROR ERROR FILTER CBC_GETSTRING 
       CBC_MAPSET CBC_NOTRANS CBC_LTRIM  CBC_RTRIM CBC_TRIM
       CBC_GETREAL CBC_GETINT PARSE BUMPNO BUMP GET_PARAMS
       GET_ONE_ARG GET_ARGS SS1 BUMPNO UNDOIT)
   ; Selection set filter closed, 2D Polylines
     (setq
      FILTER (list
                '(0 . "POLYLINE")
                '(-4 . "&")
                '(70 . 1)
                '(-4 . "<not")
                '(-4 . "&")
                '(70 . 88)
                '(-4 . "not>")
             )
   )
   ; Error Handler
   (defun ERROR (S)
      (if (not
             (member
                S
                '("Function cancelled" "console break")
             )
          )
         (princ S)
      )
      (if FH (progn (close FH) (setq FH nil)))
      (setq *error* OLD_ERROR)
      (princ)
   )
   ; Get String with default
   (defun CBC_GETSTRING (CR PRMPT DEFAULT / TEMP)
      (cond
         ((=
             ""
             (setq
                TEMP (getstring
                        CR
                        (if (/= "" DEFAULT)
                           (strcat
                              PRMPT
                              " <"
                              DEFAULT
                              ">: "
                           )
                           (strcat PRMPT ": ")
                        )
                     )
             )
          )
            DEFAULT
         )
         (t TEMP)
      )
   )
    ; Apply func to every entity in ss
   (defun CBC_MAPSET (FUNC SS / N ENAME)
      (if SS
         (progn
            (setq FUNC (eval FUNC) N (sslength SS))
            (while (> N 0)
               (setq N (1- N) ENAME (ssname SS N))
               (FUNC ENAME)
            )
         )
      )
   )
   ; Disallow transparent invocation of routine.
   (defun CBC_NOTRANS ()
      (cond
         ((zerop (logand (getvar "cmdactive") (+ 1 2 8))))
         ((alert
             "This command may not be invoked transparently."
          )
         )
      )
   )
      ; Trim Leading Spaces from string
   (defun CBC_LTRIM (ASTRING)
      (while (wcmatch ASTRING " *")
         (setq ASTRING (substr ASTRING 2))
      )
      ASTRING
   )
   ; Trim Trailing Spaces from string
   (defun CBC_RTRIM (ASTRING)
      (while (wcmatch ASTRING "* ")
        (setq ASTRING (substr ASTRING 1 (1- (strlen ASTRING)))
         )
      )
      ASTRING
   )
   ; Trim Leading and Trailing Spaces from string
   (defun CBC_TRIM (ASTRING) (CBC_RTRIM (CBC_LTRIM ASTRING)))
   ; GetReal with Default
   (defun CBC_GETREAL (PRMPT DEFAULT)
      (setq PRMPT (strcat
                  PRMPT
                  (if DEFAULT
                     (strcat
                        " <"
                        (rtos DEFAULT 2)
                        ">"
                     )
                     ""
                  )
                  ": "
               )
      )
      (cond ((getreal PRMPT)) (DEFAULT))
   )
   ; Get Integer with default
   (defun CBC_GETINT (PRMPT DEFAULT)
      (cond
         ((getint (strcat PRMPT " <" (itoa DEFAULT) ">: ")))
         (DEFAULT
         )
      )
   )
   ; Parse parameters from acad.psf
   (defun PARSE (ALINE / PATTERN COPYTO NPARAMS PARAMS)
      ; Copy aline upto delim, strip delim from aline
      (defun COPYTO (DELIM / C TOKEN)
         (setq TOKEN "")
         (while (and
                   (/= DELIM (setq C (substr ALINE 1 1)))
                   (/= "" C)
                )
            (if (/= " " C) (setq TOKEN (strcat TOKEN C)))
            (setq ALINE (substr ALINE 2))
         )
         (setq ALINE (substr ALINE 2))
         TOKEN
      )
      ; Start of Parse
      (COPYTO "%")
      (setq PATTERN (COPYTO ",")
            NPARAMS (atoi (COPYTO ","))
            PARAMS  (list NPARAMS PATTERN (strcase PATTERN))
      )
      (while (/= "" ALINE)
         (setq
            PARAMS (cons
                      (cons
                         (COPYTO "=")
                         (read (COPYTO ","))
                      )
                      PARAMS
                   )
         )
      )
      (reverse PARAMS)
   )
   ; Entertainment
   (setq BUMPNO 0)
   (defun BUMP ()
      (setq BUMPNO (logand (1+ BUMPNO) 3))
      (princ (nth BUMPNO '("\r\\" "\r-" "\r/" "\r|"))
      )
   )
   ; Read parameters from acad.psf
   (defun GET_PARAMS (/ FNAME FULLNAME FH PARAMS LBUF PARAMS)
      (setq FNAME "ACAD.PSF")
      (cond
         ((null (setq FULLNAME (findfile FNAME)))
            (alert   (strcat FNAME " -- Can't find file.")
            )
         )
         ((null (setq FH (open FULLNAME "r")))
            (alert (strcat FNAME " -- Can't open file.")
            )
         )
         (t
            (princ (strcat "\n  Reading " FULLNAME "...\r"))
            (while (setq LBUF (read-line FH))
               (if (and
                      (= (strcase LBUF) "%@FILL")
                      (setq LBUF (read-line FH))
                   )
                  (progn
                     (BUMP)
                     (setq
                        PARAMS (cons (PARSE LBUF) PARAMS)
                     )
                  )
               )
            )
            (close FH)
         )
      )
      (reverse PARAMS)
   )
   ; Get One Arg
   (defun GET_ONE_ARG (A)
      (cond
         ((atom A) A)
         ((= 'real (type (cdr A)))
            (cons
               (car A)
               (CBC_GETREAL (strcat "\n" (car A)) (cdr A))
            )
         )
         ((= 'int (type (cdr A)))
            (cons
               (car A)
               (CBC_GETINT (strcat "\n" (car A)) (cdr A))
            )
         )
         ((t A))
      )
   )
   ; Get Args
   (defun GET_ARGS (/ AGAIN PATTERN PRMPT PATTERNS 
         OLD_PARAMS NEW_PARAMS ARGS LIST_PATTERNS)
      ; Neatly list patterns from acad.psf
      (defun LIST_PATTERNS (/ TEMP)
         (textpage)
         (setq TEMP "\n")
         (foreach P PATTERNS
            (setq TEMP (strcat TEMP P "  "))
            (if (> (strlen TEMP) 70)
               (progn
                  (princ TEMP)
                  (setq TEMP "\n")
               )
            )
         )
         (if (/= TEMP "\n") (princ TEMP))
      )
      ; start of get_args
      (setq
         PATTERNS (ACAD_STRLSORT (mapcar 'cadr PSMFILL:PARAMS))
         PRMPT    "\nPostScript fill pattern (. = none) or ?"
         AGAIN    t
      )
      (while AGAIN
       (setq PATTERN (CBC_GETSTRING nil PRMPT PSMFILL:PATTERN))
         (cond
            ((= "?" PATTERN) (LIST_PATTERNS))
            ((= "." PATTERN)
               (setq OLD_PARAMS      '("." "." 0)
                     PSMFILL:PATTERN PATTERN
                     AGAIN           nil
               )
            )
            ((setq
             OLD_PARAMS (assoc(strcase PATTERN) PSMFILL:PARAMS)
             )
               (setq PSMFILL:PATTERN (cadr OLD_PARAMS)
                     AGAIN           nil
               )
            )
            ((and
                (wcmatch PATTERN "`**")
                (setq
                   OLD_PARAMS (assoc
                                 (strcase
                                    (substr PATTERN 2)
                                 )
                                 PSMFILL:PARAMS
                              )
                )
             )
               (setq
                  PSMFILL:PATTERN (strcat
                                     "*"
                                     (cadr OLD_PARAMS)
                                  )
                  AGAIN           nil
               )
            )
            (t
               (princ
                  (strcat
                     "\nUnknown PostScript Fill Pattern: "
                     (strcase
                        PATTERN
                     )
                  )
               )
            )
         )
      )
      (if OLD_PARAMS
         (setq
            NEW_PARAMS     (mapcar 'GET_ONE_ARG OLD_PARAMS)
            PSMFILL:PARAMS 
              (subst NEW_PARAMS OLD_PARAMS PSMFILL:PARAMS)
            ARGS           (cons
                              (cadr NEW_PARAMS)
                              (mapcar
                                 'cdr
                                 (cdddr NEW_PARAMS)
                              )
                           )
         )
      )
      ARGS
   )
   ; *******Main routine.
   (setq OLD_ERROR *error* *error* ERROR)
   (setvar "cmdecho" 0)
   (setq SS1 (ssget "i" FILTER))
   (cond
      ((not (CBC_NOTRANS)))
      ((progn
          (if SS1
             (progn
                (command ".select" SS1 "")
                (princ
                   (strcat
                      (itoa (sslength SS1))
                      " selected."
                   )
                )
             )
             (setq SS1 (ssget FILTER))
          )
          (null SS1)
       )
      )
      ((and (not C:PSFILL) (not (xload "acadapp" nil)))
         (if (wcmatch (getvar "platform") "* DOS *")
            (alert "ACADAPP.EXP -- Cannot Xload file."
            )
            (alert "ACADAPP.EXE -- Cannot Xload file.")
         )
      )
      ((and
          (not PSMFILL:PARAMS)
          (not (setq PSMFILL:PARAMS (GET_PARAMS)))
       )
      )
      ((not (setq ARGS (GET_ARGS))))
      (t
         (defun ONE_PSFILL (ENAME)
            (apply 'C:PSFILL (cons ENAME ARGS))
         )
         (CBC_MAPSET 'ONE_PSFILL SS1)
      )
   )
   (setq SS1 nil *error* OLD_ERROR)
   (princ)
)
(princ
   (strcat
      "\nPSMFILL.LSP -- (c) 1995 "
      "by Looking Glass Microproducts"
   ))(princ);;end psmfill.lsp
